<?php

namespace Drupal\screenshot_one\Plugin\AiAutomatorType;

use Drupal\ai_automator\PluginBaseClasses\ExternalBase;
use Drupal\ai_automator\PluginInterfaces\AiAutomatorTypeInterface;
use Drupal\screenshot_one\ScreenshotOne;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The rules for an screenshots.
 */
class ScreenshotBase extends ExternalBase implements AiAutomatorTypeInterface, ContainerFactoryPluginInterface {

  /**
   * {@inheritDoc}
   */
  public $title = 'Screenshot One Image';

  /**
   * The screenshot one api service.
   */
  public ScreenshotOne $screenshotOne;

  /**
   * Construct an image field.
   *
   * @param array $configuration
   *   Inherited configuration.
   * @param string $plugin_id
   *   Inherited plugin id.
   * @param mixed $plugin_definition
   *   Inherited plugin definition.
   * @param \Drupal\ai_automator_screenshot\ScreenshotOne $screenshotOne
   *   The Screenshot API service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ScreenshotOne $screenshotOne,
  ) {
    $this->screenshotOne = $screenshotOne;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('screenshot_one.api'),
    );
  }

  /**
   * {@inheritDoc}
   */
  public function needsPrompt() {
    return FALSE;
  }

  /**
   * {@inheritDoc}
   */
  public function advancedMode() {
    return FALSE;
  }

  /**
   * {@inheritDoc}
   */
  public function allowedInputs() {
    return [
      'link',
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function helpText() {
    return $this->t("Screenshot an images based on a url.");
  }

  /**
   * {@inheritDoc}
   */
  public function placeholderText() {
    return "";
  }

  /**
   * {@inheritDoc}
   */
  public function extraFormFields(ContentEntityInterface $entity, FieldDefinitionInterface $fieldDefinition, FormStateInterface $formState, array $defaultValues = []) {
    $form['automator_screenshot_width'] = [
      '#type' => 'number',
      '#title' => 'Screenshot Width',
      '#required' => TRUE,
      '#description' => $this->t('The width of the screenshot'),
      '#default_value' => $defaultValues['automator_screenshot_width'] ?? 1280,
    ];

    $form['automator_screenshot_height'] = [
      '#type' => 'number',
      '#title' => 'Screenshot Height',
      '#required' => TRUE,
      '#description' => $this->t('The height of the screenshot'),
      '#default_value' => $defaultValues['automator_screenshot_height'] ?? 1280,
    ];

    $form['automator_screenshot_delay'] = [
      '#type' => 'number',
      '#title' => 'Screenshot Delay',
      '#required' => TRUE,
      '#description' => $this->t('Delay of taking the screenshot'),
      '#default_value' => $defaultValues['automator_screenshot_delay'] ?? 0,
    ];

    $form['automator_screenshot_full_page'] = [
      '#type' => 'checkbox',
      '#title' => 'Fullpage',
      '#description' => $this->t('Take a full page screenshot'),
      '#default_value' => $defaultValues['automator_screenshot_full_page'] ?? FALSE,
    ];

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function generate(ContentEntityInterface $entity, FieldDefinitionInterface $fieldDefinition, array $automatorConfig) {
    $values = [];
    if (!empty($entity->{$automatorConfig['base_field']}->uri)) {
      foreach ($entity->get($automatorConfig['base_field']) as $wrapperEntity) {
        $link = $wrapperEntity->uri;
        // We generate later, so we don't run out of memory.
        $values[] = $link;
      }
    }
    return $values;
  }

  /**
   * {@inheritDoc}
   */
  public function verifyValue(ContentEntityInterface $entity, $value, FieldDefinitionInterface $fieldDefinition, $automatorConfig) {
    if (filter_var($value, FILTER_VALIDATE_URL)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * {@inheritDoc}
   */
  public function storeValues(ContentEntityInterface $entity, array $values, FieldDefinitionInterface $fieldDefinition, $automatorConfig) {
    $fileHelper = $this->getFileHelper();
    $params = $this->getParams($automatorConfig);

    // Transform string to boolean.
    $fileEntities = [];

    // Successful counter, to only download as many as max.
    $successFul = 0;
    foreach ($values as $value) {
      // Get filename from previewURL.
      $fileName = 'screenshot_' . md5($value) . '.jpg';
      $path = $fileHelper->createFilePathFromFieldConfig($fileName, $fieldDefinition, $entity);
      $fileString = $this->screenshotOne->screenshotUrl($value, $params);
      $file = $fileHelper->generateFileFromBinary($fileString, $path);
      // If we can save, we attach it.
      if ($file) {
        // Get resolution.
        $resolution = getimagesize($file->uri->value);
        // Add to the entities list.
        $fileEntities[] = [
          'target_id' => $file->id(),
          'alt' => $this->t('Screenshot of @url', ['@url' => $value]),
          'title' => $this->t('Screenshot of @url', ['@url' => $value]),
          'width' => $resolution[0],
          'height' => $resolution[1],
        ];

        $successFul++;
        // If we have enough images, give up.
        if ($successFul == $fieldDefinition->getFieldStorageDefinition()->getCardinality()) {
          break;
        }
      }
    }

    // Then set the value.
    $entity->set($fieldDefinition->getName(), $fileEntities);
  }

  /**
   * Generate params from config.
   *
   * @param array $specs
   *   The config spec.
   *
   * @return array
   *   The params to screenshot.
   */
  protected function getParams(array $specs) {
    $params = [];
    foreach ($specs as $key => $val) {
      if (strpos($key, 'screenshot_') === 0) {
        $params[str_replace('screenshot_', '', $key)] = $val;
      }
    }

    return $params;
  }
}
